__all__ = ["Crawler", "CrawlerCollection", "NPCore"]

from random import choice as random_choice, uniform as random_float
from threading import Thread
from types import MethodType
from typing import Callable, List, Optional, Set, Union
from weakref import ReferenceType, WeakMethod

from nichtparasoup.core.image import Image, ImageCollection, ImageUri
from nichtparasoup.core.imagecrawler import BaseImageCrawler

_CrawlerWeight = Union[int, float]  # constraint: > 0


class _Blacklist(Set[ImageUri]):
    pass


_IsImageAddable = Callable[[Image], bool]

_OnImageAdded = Callable[[Image], None]

_OnFill = Callable[["Crawler", int], None]

_FILLUP_TIMEOUT_DEFAULT = 1.0


class Crawler(object):

    def __init__(self, imagecrawler: BaseImageCrawler, weight: _CrawlerWeight,
                 is_image_addable: Optional[_IsImageAddable] = None,
                 on_image_added: Optional[_OnImageAdded] = None) -> None:  # pragma: no cover
        if weight <= 0:
            raise ValueError('weight <= 0')
        self.imagecrawler = imagecrawler
        self.weight = weight
        self.images = ImageCollection()
        self._is_image_addable_wr = None  # type: Optional[ReferenceType[_IsImageAddable]]
        self._image_added_wr = None  # type: Optional[ReferenceType[_OnImageAdded]]
        self.set_is_image_addable(is_image_addable)
        self.set_image_added(on_image_added)

    def set_is_image_addable(self, is_image_addable: Optional[_IsImageAddable]) -> None:
        t_is_image_addable = type(is_image_addable)
        if None is is_image_addable:
            self._is_image_addable_wr = None
        elif MethodType is t_is_image_addable:
            self._is_image_addable_wr = WeakMethod(is_image_addable)  # type: ignore
        else:
            raise Exception('type {} not supported, yet'.format(t_is_image_addable))
        # TODO: add function and other types - and write proper tests for it

    def get_is_image_addable(self) -> Optional[_IsImageAddable]:
        return self._is_image_addable_wr() if self._is_image_addable_wr else None

    def set_image_added(self, image_added: Optional[_OnImageAdded]) -> None:
        t_image_added = type(image_added)
        if None is image_added:
            self._image_added_wr = None
        elif MethodType is t_image_added:
            self._image_added_wr = WeakMethod(image_added)  # type: ignore
        else:
            raise Exception('type {} not supported, yet'.format(t_image_added))
        # TODO: add function and other types - and write proper tests for it

    def get_image_added(self) -> Optional[_OnImageAdded]:
        return self._image_added_wr() if self._image_added_wr else None

    def reset(self) -> None:
        self.images.clear()
        self.imagecrawler.reset()

    def crawl(self) -> int:
        is_image_addable = self.get_is_image_addable()
        image_added = self.get_image_added()
        images_crawled = self.imagecrawler.crawl()
        for image_crawled in images_crawled:
            addable = is_image_addable(image_crawled) if is_image_addable else True
            if not addable:
                continue  # for
            self.images.add(image_crawled)
            if image_added:
                image_added(image_crawled)
        return len(images_crawled)

    def fill_up_to(self, to: int,
                   filled_by: Optional[_OnFill] = None,
                   timeout: float = _FILLUP_TIMEOUT_DEFAULT) -> None:
        from time import sleep
        while len(self.images) < to:
            refilled = self.crawl()
            if filled_by:
                filled_by(self, refilled)
            if 0 == refilled:
                break  # while
            if len(self.images) < to and timeout > 0:
                # be nice, give the site some rest after crawling
                sleep(timeout)

    def get_random_image(self) -> Optional[Image]:
        if not self.images:
            return None
        image = random_choice(list(self.images))
        return image

    def pop_random_image(self) -> Optional[Image]:
        image = self.get_random_image()
        if image:
            self.images.discard(image)
        return image


class CrawlerCollection(List[Crawler]):

    def _random_weight(self) -> _CrawlerWeight:
        return random_float(0, sum(crawler.weight for crawler in self))

    def get_random(self) -> Optional[Crawler]:
        cum_weight_goal = self._random_weight()
        # IDEA: cum_weight_goal == 0 is an edge case and could be handled if needed ...
        cum_weight = 0  # type: _CrawlerWeight
        for crawler in self:
            cum_weight += crawler.weight
            if cum_weight >= cum_weight_goal:
                return crawler
        return None


class NPCore(object):

    def __init__(self) -> None:  # pragma: no cover
        self.crawlers = CrawlerCollection()
        self.blacklist = _Blacklist()

    def _is_image_not_in_blacklist(self, image: Image) -> bool:
        # must be compatible to: _IsImageAddable
        return image.uri not in self.blacklist

    def _add_image_to_blacklist(self, image: Image) -> None:
        # must be compatible to: _OnImageAdded
        if not image.is_generic:
            self.blacklist.add(image.uri)

    def has_imagecrawler(self, imagecrawler: BaseImageCrawler) -> bool:
        return imagecrawler in (crawler.imagecrawler for crawler in self.crawlers)

    def add_imagecrawler(self, imagecrawler: BaseImageCrawler, weight: _CrawlerWeight) -> None:
        self.crawlers.append(Crawler(
            imagecrawler, weight,
            self._is_image_not_in_blacklist, self._add_image_to_blacklist
        ))

    def fill_up_to(self, to: int, on_refill: Optional[_OnFill], timeout: float = _FILLUP_TIMEOUT_DEFAULT) -> None:
        fill_treads = list()  # type: List[Thread]
        for crawler in self.crawlers:
            fill_tread = Thread(target=crawler.fill_up_to, args=(to, on_refill, timeout), daemon=True)
            fill_treads.append(fill_tread)
            fill_tread.start()
        for fill_tread in fill_treads:
            fill_tread.join()

    def reset(self) -> int:
        reset_treads = list()  # type: List[Thread]
        for crawler in self.crawlers.copy():
            reset_tread = Thread(target=crawler.reset, daemon=True)
            reset_treads.append(reset_tread)
            reset_tread.start()
        blacklist_len = len(self.blacklist)
        self.blacklist.clear()
        for reset_tread in reset_treads:
            reset_tread.join()
        return blacklist_len



__all__ = ["FileFetcher", "ImageCrawlerLoaderTest"]

from abc import ABC, abstractmethod
from collections import OrderedDict
from http.client import HTTPResponse
from os.path import join as path_join
from pathlib import Path
from typing import Dict, Tuple, Type, Union
from unittest import TestCase
from urllib.parse import parse_qs, urlencode, urlparse, urlunparse
from urllib.response import addinfourl

from nichtparasoup.core.imagecrawler import BaseImageCrawler, RemoteFetcher
from nichtparasoup.imagecrawler import get_imagecrawlers


class FileFetcher(RemoteFetcher):
    r"""
    A file fetcher that can be used for testing with local files.

    URI are modified so query params are sorted - which makes same URL unique.

    Remember the FileSystem restrictions:
    * some FS do not support CaseSensitivity
    * some FS do not allow these characters: \/:*?"<>|

    """

    def __init__(self, known_files: Dict[str, str], base_dir: str = '') -> None:  # pragma: no cover
        super().__init__()
        self._known = {
            self.__class__._uri_sort_query(k): self._build_uri(v, base_dir)
            for k, v
            in known_files.items()
        }  # type: Dict[str, str]

    @classmethod
    def _build_uri(cls, file: str, base_dir: str = '') -> str:
        file_path = Path(path_join(base_dir, file) if base_dir else file)
        cls._test_path(file_path)
        return file_path.as_uri()

    @staticmethod
    def _test_path(file_path: Path) -> None:
        if not file_path.is_absolute():
            raise FileNotFoundError('Path not absolute: {!r}'.format(file_path))
        if not file_path.is_file():
            raise FileNotFoundError('Not a file: {!r}'.format(file_path))
        # test if readable. will raise errors on its own
        file_path.open('r').close()

    @classmethod
    def _uri_sort_query(cls, uri: str) -> str:
        scheme, netloc, path, params, query, fragment = urlparse(uri)
        if query == '':
            query_sorted = query
        else:
            query_dict = parse_qs(query, keep_blank_values=True)
            query_dict_sorted = OrderedDict((k, query_dict[k]) for k in sorted(query_dict))
            query_sorted = urlencode(query_dict_sorted, doseq=True)
        uri_sorted = urlunparse((scheme, netloc, path, params, query_sorted, fragment))
        return uri_sorted

    def _get_file_uri(self, uri: str) -> str:
        _, _, url, params, query, fragment = urlparse(uri)
        uri_abs = urlunparse(('', '', url, params, query, fragment))
        uri_sorted = self.__class__._uri_sort_query(uri_abs)
        known = self._known.get(uri_sorted)
        if not known:
            raise FileNotFoundError('uri unexpected: {}'.format(uri_sorted))
        return known

    @staticmethod
    def _valid_uri(uri: str) -> bool:
        scheme, _, _, _, _, _ = urlparse(uri)
        return scheme == 'file'

    def get_stream(self, uri: str) -> Tuple[Union[HTTPResponse, addinfourl], str]:
        response, _ = super().get_stream(self._get_file_uri(uri))
        return response, uri


class ImageCrawlerLoaderTest(TestCase, ABC):
    """
    Helper fo testing if the loader finds the ImageCrawler plugin properly.

    Just implement the abstract properties ``ic_name`` and ``ic_class``
    and call the ``check`` method in a test.
    """

    @property
    @abstractmethod
    def ic_name(self) -> str:  # pragma: no cover
        """
        Return the intended ImageCrawler's name.
        That's basically the name you chose in the EntryPoint.

        Example implementation:
            return "MyImageCrawler"
        """
        raise NotImplementedError()

    @property
    @abstractmethod
    def ic_class(self) -> Type[BaseImageCrawler]:  # pragma: no cover
        """
        the class of your ImageCrawler.

        Example implementation:
            return MyImageCrawler
        """
        raise NotImplementedError()

    def check(self) -> None:
        self.check_get_imagecrawler_class()
        self.check_get_imagecrawler_name()

    def check_get_imagecrawler_class(self) -> None:
        # act
        imagecrawler_class = get_imagecrawlers().get_class(self.ic_name)
        # assert
        self.assertIs(imagecrawler_class, self.ic_class)

    def check_get_imagecrawler_name(self) -> None:
        # act
        imagecrawler_name = get_imagecrawlers().get_name(self.ic_class)
        # assert
        self.assertEqual(imagecrawler_name, self.ic_name)



import unittest

from nichtparasoup.core.image import Image, ImageCollection


class ImageTest(unittest.TestCase):

    def test_uri_is_hash(self) -> None:
        # arrange
        uri = 'test'
        image = Image(uri=uri, source='test')
        # assert
        self.assertEqual(hash(uri), hash(image))

    def test_uri_makes_equal(self) -> None:
        # arrange
        uri = 'test'
        image1 = Image(uri=uri, source='test1')
        image2 = Image(uri=uri, source='test2')
        # assert
        self.assertEqual(image1, image2)

    def test_unequal_other_types(self) -> None:
        # arrange
        image = Image(uri="testA", source='test')
        other_types = [None, True, 23, 4.2, "", [], (), {}, self]  # type: ignore
        # assert
        for other_type in other_types:
            self.assertNotEqual(image, other_type)

    def test_equal(self) -> None:
        # arrange
        image1 = Image(uri="testA", source='testA')
        image2 = Image(uri="testA", source='testA')
        image3 = Image(uri="testB", source='testB', is_generic=True)
        image4 = Image(uri="testB", source='testB', is_generic=True)
        # assert
        self.assertEqual(image1, image1)
        self.assertEqual(image1, image2)
        self.assertEqual(image3, image3)
        self.assertNotEqual(image3, image4)

    def test_remove_nongeneric_from_container(self) -> None:
        # arrange
        image1 = Image(uri="testA", source='testA', is_generic=True)
        image2 = Image(uri="testA", source='testA', is_generic=True)
        images = ImageCollection()
        images.add(image1)
        # act
        images.discard(image2)
        # assert
        self.assertEqual(1, len(images))
        self.assertIn(image1, images)



from abc import ABC
from typing import Any, Dict

from nichtparasoup.core.image import ImageCollection
from nichtparasoup.core.imagecrawler import BaseImageCrawler, ImageCrawlerConfig, ImageCrawlerInfo


class StandaloneLoadableImageCrawlerA(object):
    pass


class BaseLoadableImageCrawlerA(BaseImageCrawler, ABC):
    pass


class LoadableImageCrawlerA(BaseImageCrawler):
    @classmethod
    def info(cls) -> ImageCrawlerInfo:
        pass

    @classmethod
    def check_config(cls, config: Dict[Any, Any]) -> ImageCrawlerConfig:
        pass

    def _reset(self) -> None:
        pass

    def _crawl(self) -> ImageCollection:
        pass


class LoadableImageCrawlerB(LoadableImageCrawlerA):
    pass



__all__ = ["Pr0gramm"]

from json import loads as json_loads
from typing import Any, Dict, Optional
from urllib.parse import urlencode, urljoin

from nichtparasoup.imagecrawler import (
    BaseImageCrawler, Image, ImageCollection, ImageCrawlerConfig, ImageCrawlerInfo, RemoteFetcher,
)


class Pr0gramm(BaseImageCrawler):

    def __init__(self, **config: Any) -> None:  # pragma: no cover
        super().__init__(**config)
        self._older = None  # type: Optional[int]
        self._remote_fetcher = RemoteFetcher()

    @classmethod
    def info(cls) -> ImageCrawlerInfo:
        return ImageCrawlerInfo(
            description='A Crawler for https://pr0gramm.com',
            config=dict(
                promoted='Boolean. Search only top("beliebt") voted content? Otherwise search anything("neu").',
                tags='Filter. None, or a string that starts with "!" - see https://pr0gramm.com/new/2782197',
            ),
            icon_url='https://pr0gramm.com/media/pr0gramm-favicon.png',
        )

    @staticmethod
    def __check_config_tags(tags: Optional[str]) -> Optional[str]:
        if tags is None:
            return None
        if type(tags) is str:
            tags = tags.strip()
            if not tags.startswith('!'):
                raise ValueError('tags {!r} must start with "!"'.format(tags))
            if not len(tags) > 1:
                raise ValueError('tags {!r} is empty'.format(tags))
            return tags
        raise TypeError('tags {!r} is not str or None'.format(tags))

    @classmethod
    def check_config(cls, config: Dict[Any, Any]) -> ImageCrawlerConfig:
        promoted = config['promoted'] if 'promoted' in config else True  # type: bool
        if type(promoted) is not bool:
            raise TypeError('promoted {!r} is not bool'.format(promoted))
        tags = config['tags'] if 'tags' in config else None
        tags = cls.__check_config_tags(tags)
        return ImageCrawlerConfig(
            promoted=promoted,
            tags=tags,
        )

    __API_GET_URL = 'https://pr0gramm.com/api/items/get'

    @classmethod
    def _get_api_uri(cls, *,
                     flags: int, promoted: bool,
                     tags: Optional[str] = None, older: Optional[int] = None) -> str:
        """
        :param flags: BitSet. sfw=1, nsfw=2, nsfl=4
        :param promoted: Search top("beliebt") only? - Otherwise search all("neu").
        :param tags: None, or a string that starts with "!" - see https://pr0gramm.com/new/2782197
        :param older: page through the search
        """
        params = dict(
            flags=str(flags),
            promoted=('1' if promoted else '0'),
            tags='!{} -"video"'.format('({})'.format(tags.lstrip('!')) if tags else '')
        )
        if older:
            params['older'] = str(older)
        return cls.__API_GET_URL + '?' + urlencode(params)

    def _reset(self) -> None:
        self._older = None

    __IMG_BASE_URL = 'https://img.pr0gramm.com/'
    __POST_BASE_URL = 'https://pr0gramm.com/new/'

    def _crawl(self) -> ImageCollection:
        images = ImageCollection()
        promoted = self._config['promoted']
        api_uri = self._get_api_uri(
            flags=1,
            promoted=promoted,
            tags=self._config.get('tags', None),
            older=self._older)
        response_raw, api_uri = self._remote_fetcher.get_string(api_uri)
        response = json_loads(response_raw)
        for item in response['items']:
            images.add(Image(
                uri=urljoin(self.__IMG_BASE_URL, str(item['image'])),
                source=urljoin(self.__POST_BASE_URL, str(item['id'])),
                width=item.get('width'),
                height=item.get('height'),
            ))
        if response['atEnd']:
            self.reset()
        else:
            self._older = response['items'][-1]['promoted' if promoted else 'id'] or None
        return images



__all__ = ["ImageCrawlerConfig", "BaseImageCrawler", "ImageCrawlerInfo", "RemoteFetcher", "ImageRecognizer"]

from abc import ABC, abstractmethod
from http.client import HTTPResponse
from re import IGNORECASE as RE_IGNORECASE, compile as re_compile
from threading import Lock
from typing import Any, Dict, Optional, Pattern, Tuple, Union
from urllib.parse import urlparse
from urllib.request import Request, urlopen
from urllib.response import addinfourl

from nichtparasoup._internals import _log
from nichtparasoup.core.image import ImageCollection

_ImageCrawlerConfigKey = str


class ImageCrawlerInfo(object):
    """
    ImageCrawler's Info.

    .. seealso:: :method:`BaseImageCrawler.info()`

    """

    def __init__(self, *,
                 description: str, long_description: Optional[str] = None,
                 config: Optional[Dict[_ImageCrawlerConfigKey, str]] = None,
                 icon_url: Optional[str] = None,
                 **more: Any) -> None:  # pragma: no cover
        """
        :param description: short description
        :param long_description: long description
        :param config: config description(s)
        :param icon_url: url to an icon-like image. maybe the favicon. use 'https://' if possible!
        :param more: more to save - planned to the future
        """
        self.description = description
        self.long_description = long_description
        self.config = config
        self.icon_url = icon_url
        del more  # currently not stored, but planned for the future


class ImageCrawlerConfig(Dict[_ImageCrawlerConfigKey, Any]):
    pass


class BaseImageCrawler(ABC):

    def __init__(self, **config: Any) -> None:  # pragma: no cover
        self._config = self.check_config(config)  # intended to be immutable from now on
        self._reset_before_next_crawl = True
        self._crawl_lock = Lock()
        _log('debug', 'crawler initialized: {!r}'.format(self))

    def __repr__(self) -> str:  # pragma: no cover
        return '<{0.__module__}.{0.__name__} {1!r}>'.format(type(self), self.get_config())

    def __eq__(self, other: Any) -> bool:
        if type(self) is type(other):
            other_imagecrawler = other  # type: BaseImageCrawler
            return self._config == other_imagecrawler._config
        return False

    def get_config(self) -> ImageCrawlerConfig:
        """
        Get all *public* information from the config

        For internal access to the config using `self._config` is encouraged

        """
        return ImageCrawlerConfig({
            k: v
            for k, v
            in self._config.items()
            if not k.startswith('_')
        })

    def reset(self) -> None:
        self._reset_before_next_crawl = True
        _log('debug', 'crawler reset planned for {!r}'.format(self))

    def crawl(self) -> ImageCollection:  # pragma: no cover
        with self._crawl_lock:
            try:
                if self._reset_before_next_crawl:
                    _log('debug', 'crawler resetting {!r}'.format(self))
                    self._reset()
                    self._reset_before_next_crawl = False
                _log('debug', 'crawling started {!r}'.format(self))
                crawled = self._crawl()
                _log('debug', 'crawling finished {!r}'.format(self))
                return crawled
            except Exception:
                _log('exception', 'caught an error during crawling {!r}'.format(self))
                return ImageCollection()

    @classmethod
    @abstractmethod
    def info(cls) -> ImageCrawlerInfo:  # pragma: no cover
        """
        Get info of the crawler

        example implementation:
            return ImageCrawlerInfo(
                description='Textual description what this ImageCrawler does. Where it gets the images from.',
                long_description=textwrap.dedent('''
                You may want write a long description of the ImageCrawler. Feel free to do so.
                This is the place where you can do this.
                ''').strip(),
                config=dict(
                    # leave the dict empty, if there is nothing to configure. or just don't pass a config at all.
                    param1='purpose & meaning of param1',
                    # ...
                    paramN='purpose & meaning of paramN',
                ),
                icon_url='https://my.imagesource.net/favicon.png'
            )

        """
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def check_config(cls, config: Dict[Any, Any]) -> ImageCrawlerConfig:  # pragma: no cover
        """
        This function is intended to check if a config is valid and to strip unused config.

        When implementing:
            Check if any config is viable. if not raise ValueError or TypeError or KeyError
            or whatever Error.
            Return the viable config for this crawler instance.

        Example implementation:
            height = config["height"]  # will raise KeyError automatically
            if type(height) is not int:
                raise TypeError("height {} is not int".format(height))
            if height <= 0:
                raise ValueError("height {} <= 0".format(width))

        """
        raise NotImplementedError()

    @abstractmethod
    def _reset(self) -> None:  # pragma: no cover
        """
        This function is intended to reset the crawler to restart at front
        """
        raise NotImplementedError()

    @abstractmethod
    def _crawl(self) -> ImageCollection:  # pragma: no cover
        """
        This function is intended to find and fetch ImageURIs
        """
        raise NotImplementedError()


class RemoteFetcher(object):
    _HEADERS_DEFAULT = {
        'User-Agent': 'NichtParasoup',
    }

    def __init__(self, timeout: float = 10.0, headers: Optional[Dict[str, str]] = None) -> None:  # pragma: no cover
        self._timeout = timeout
        self._headers = self.__class__._HEADERS_DEFAULT.copy()
        if headers:
            self._headers.update(headers)

    @staticmethod
    def _valid_uri(uri: str) -> bool:
        (scheme, _, _, _, _, _) = urlparse(uri)
        return scheme in {'http', 'https'}

    def get_stream(self, uri: str) -> Tuple[Union[HTTPResponse, addinfourl], str]:
        if not self._valid_uri(uri):
            raise ValueError('not remote: ' + uri)
        _log('debug', 'fetch remote {!r} in {}s with {!r}'.format(
            uri, self._timeout, self._headers))
        request = Request(uri, headers=self._headers)
        try:
            response = urlopen(request, timeout=self._timeout)  # type: Union[HTTPResponse, addinfourl]
        except Exception as e:
            _log('debug', 'caught error on fetch remote {!r}'.format(uri), exc_info=True)
            raise e
        actual_uri = response.geturl()  # after following redirects ...
        return response, actual_uri

    def get_bytes(self, uri: str) -> Tuple[bytes, str]:
        response, actual_uri = self.get_stream(uri)
        return response.read(), actual_uri

    def get_string(self, uri: str, charset_fallback: str = 'UTF-8') -> Tuple[str, str]:
        response, actual_uri = self.get_stream(uri)
        charset = str(response.info().get_param('charset', charset_fallback))
        return response.read().decode(charset), actual_uri


class ImageRecognizer(object):
    _PATH_RE = re_compile(r'.+\.(?:jpeg|jpg|png|gif|svg)(?:[?#].*)?$', flags=RE_IGNORECASE)  # type: Pattern[str]

    def path_is_image(self, uri: str) -> bool:
        return self._PATH_RE.match(uri) is not None



__all__ = ["ConfigFileTest", "ImageCrawlerProbeCrawlError"]

from time import sleep
from typing import List
from unittest import TestCase

from nichtparasoup.config import get_imagecrawler, parse_yaml_file
from nichtparasoup.core.imagecrawler import BaseImageCrawler


class ConfigFileTest(TestCase):

    def validate(self, file: str) -> None:
        config = parse_yaml_file(file)
        self.assertIsInstance(config, dict)
        imagecrawlers = list()  # type: List[BaseImageCrawler]
        for crawler_config in config['crawlers']:
            imagecrawler = get_imagecrawler(crawler_config)
            self.assertNotIn(imagecrawler, imagecrawlers, msg='Duplicate ImageCrawler')
            imagecrawlers.append(imagecrawler)

    @staticmethod
    def _probe_crawl(imagecrawler: BaseImageCrawler) -> None:  # pragma: no cover
        try:
            imagecrawler._crawl()
        except Exception as e:
            raise ImageCrawlerProbeCrawlError(imagecrawler) from e

    def probe(self, file: str) -> None:  # pragma: no cover
        config = parse_yaml_file(file)
        self.assertIsInstance(config, dict)
        for crawler_config in config['crawlers']:
            imagecrawler = get_imagecrawler(crawler_config)
            self._probe_crawl(imagecrawler)
            sleep(0.023)  # do not be too greedy


class ImageCrawlerProbeCrawlError(Exception):

    def __init__(self, imagecrawler: BaseImageCrawler) -> None:  # pragma: no cover
        self._imagecrawler = imagecrawler

    def __str__(self) -> str:  # pragma: no cover
        return 'Probe-crawl failed for {!r}'.format(self._imagecrawler)



import unittest

from nichtparasoup.core.imagecrawler import ImageRecognizer


class BaseImageCrawlerPathIsImageTest(unittest.TestCase):

    def test_path_is_image(self) -> None:
        # arrange
        recognizer = ImageRecognizer()
        image_file_extensions = ('jpg', 'jpeg', 'gif', 'png', 'svg')
        # act & assert
        for image_file_extension in image_file_extensions:
            image_file_path = 'foo.' + image_file_extension
            self.assertTrue(recognizer.path_is_image(image_file_path), image_file_path)
            self.assertTrue(recognizer.path_is_image(image_file_path + '?foo'), image_file_path)
            self.assertTrue(recognizer.path_is_image(image_file_path + '#bar'), image_file_path)
            self.assertTrue(recognizer.path_is_image(image_file_path + '?foo#bar'), image_file_path)

    def test_path_is_not_image(self) -> None:
        # arrange
        recognizer = ImageRecognizer()
        not_image_file_extensions = ('', '/', '.html', '.js', '.css')
        # act & assert
        for not_image_file_extension in not_image_file_extensions:
            not_image_file_path = 'foo' + not_image_file_extension
            self.assertFalse(recognizer.path_is_image(not_image_file_path), not_image_file_path)
            self.assertFalse(recognizer.path_is_image(not_image_file_path + '?foo'), not_image_file_path)
            self.assertFalse(recognizer.path_is_image(not_image_file_path + '#bar'), not_image_file_path)
            self.assertFalse(recognizer.path_is_image(not_image_file_path + '?foo#bar'), not_image_file_path)
